@@ -4,8 +4,8 @@ from bleak_winrt.windows.devices.bluetooth.genericattributeprofile import (
     GattDeviceService,
 )
 
-from bleak.backends.service import BleakGATTService
-from bleak.backends.winrt.characteristic import BleakGATTCharacteristicWinRT
+from ..service import BleakGATTService
+from ..winrt.characteristic import BleakGATTCharacteristicWinRT
 
 
 class BleakGATTServiceWinRT(BleakGATTService):
